@@ -26,14 +26,14 @@ pub enum Error {
 impl fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
-            &Error::ParseCannotOpenFile { ref src_path } => {
+            Error::ParseCannotOpenFile { ref src_path } => {
                 write!(f, "Parsing crate: cannot open file `{}`.", src_path)
             }
-            &Error::ParseSyntaxError {
+            Error::ParseSyntaxError {
                 ref src_path,
                 ref error,
             } => write!(f, "Parsing crate:`{}`:\n{:?}", src_path, error),
-            &Error::LexError {
+            Error::LexError {
                 ref src_path,
                 ref line,
             } => write!(f, "{}:{}: Lexing error", src_path, line + 1),
@@ -74,20 +74,49 @@ fn expand_sub_rust_macro(input: String, mut t: ExpandSubMacroType) -> Result<Str
     let mut result = input;
     let mut extra_decl = String::new();
     let mut search_index = 0;
-    while let Some(index) = result[search_index..].find("rust!") {
-        let (rust_invocation, begin, end) = {
+
+    loop {
+        let (begin, end, line) = {
+            let mut begin = 0;
             let mut cursor = new_cursor(&result);
-            cursor = cursor.advance(search_index + index);
-            search_index = cursor.off as usize + 4;
+            cursor.advance(search_index);
+            while !cursor.is_empty() {
+                cursor = skip_whitespace(cursor);
+                let r = skip_literal(cursor)?;
+                cursor = r.0;
+                if r.1 {
+                    continue;
+                }
+                if cursor.is_empty() {
+                    break;
+                }
+                if let Ok((cur, ident)) = symbol(cursor) {
+                    begin = cursor.off as usize;
+                    cursor = cur;
+                    if ident != "rust" {
+                        continue;
+                    }
+                } else {
+                    cursor = cursor.advance(1);
+                    continue;
+                }
+                cursor = skip_whitespace(cursor);
+                if !cursor.starts_with("!") {
+                    continue;
+                }
+                break;
+            }
+            if cursor.is_empty() {
+                return Ok(extra_decl + &result);
+            }
             let end = find_delimited((find_delimited(cursor, "(")?.0).advance(1), ")")?.0;
-            let input: ::proc_macro2::TokenStream = (&cursor.rest
-                [..(end.off + 1 - cursor.off) as usize])
-                .parse()
-                .map_err(|_| LineError(cursor.line, "TokenStream parse error".into()))?;
-            let rust_invocation = ::syn::parse2::<RustInvocation>(input)
-                .map_err(|e| LineError(cursor.line, e.to_string()))?;
-            (rust_invocation, cursor.off as usize, end.off as usize + 1)
+            (begin, end.off as usize + 1, cursor.line)
         };
+        let input: ::proc_macro2::TokenStream = result[begin..end]
+            .parse()
+            .map_err(|_| LineError(line, "TokenStream parse error".into()))?;
+        let rust_invocation =
+            ::syn::parse2::<RustInvocation>(input).map_err(|e| LineError(line, e.to_string()))?;
         let fn_name = match t {
             ExpandSubMacroType::Lit => {
                 extra_decl.push_str(&format!("extern \"C\" void {}();\n", rust_invocation.id));
@@ -147,9 +176,8 @@ fn expand_sub_rust_macro(input: String, mut t: ExpandSubMacroType) -> Result<Str
         };
         // add the invocation of call where the rust! macro used to be.
         result.insert_str(begin, &fn_call);
+        search_index = begin + fn_call.len();
     }
-
-    return Ok(extra_decl + &result);
 }
 
 #[test]
@@ -168,6 +196,12 @@ fn test_expand_sub_rust_macro() {
         ExpandSubMacroType::Lit,
     );
     assert_eq!(x.unwrap(), "extern \"C\" void xxx();\nextern \"C\" void yyy();\n{ hello( reinterpret_cast<void (*)()>(xxx)(), reinterpret_cast<void (*)()>(yyy)(); ) }");
+
+    let s = "{ /* rust! */  /* rust!(xxx [] { 1 }) */ }".to_owned();
+    assert_eq!(
+        expand_sub_rust_macro(s.clone(), ExpandSubMacroType::Lit).unwrap(),
+        s
+    );
 }
 
 #[path = "strnom.rs"]
@@ -209,7 +243,7 @@ fn skip_literal(mut input: Cursor) -> PResult<bool> {
         input = input.advance(q + 1);
         return raw_string(input).map(|x| (x.0, true));
     }
-    return Ok((input, false));
+    Ok((input, false))
 }
 
 fn new_cursor(s: &str) -> Cursor {
@@ -270,7 +304,7 @@ fn find_delimited<'a>(mut input: Cursor<'a>, needle: &str) -> PResult<'a, ()> {
         }
         input = input.advance(1);
     }
-    return Err(LexError { line: input.line });
+    Err(LexError { line: input.line })
 }
 
 #[test]
@@ -303,11 +337,15 @@ fn test_cursor_advance() -> Result<(), LexError> {
 }
 
 fn line_directive(path: &PathBuf, cur: Cursor) -> String {
-    let mut line = format!("#line {} {:?}\n", cur.line + 1, path);
+    let mut line = format!(
+        "#line {} \"{}\"\n",
+        cur.line + 1,
+        path.to_string_lossy().replace('\\', "\\\\")
+    );
     for _ in 0..cur.column {
         line.push(' ');
     }
-    return line;
+    line
 }
 
 #[derive(Default)]
@@ -471,17 +509,18 @@ impl Parser {
         match ::syn::parse2::<Macro>(input).map_err(|e| LineError(x.line, e.to_string()))? {
             Macro::Closure(mut c) => {
                 c.callback_offset = self.callbacks_count;
-                c.body_str = line_directive(&self.current_path, begin)
-                    + &expand_sub_rust_macro(
-                        extracted.to_string(),
-                        ExpandSubMacroType::Closure(&mut self.callbacks_count),
-                    ).map_err(|e| e.add_line(begin.line))?;
+                c.body_str = line_directive(&self.current_path, begin) + &expand_sub_rust_macro(
+                    extracted.to_string(),
+                    ExpandSubMacroType::Closure(&mut self.callbacks_count),
+                ).map_err(|e| e.add_line(begin.line))?;
                 self.closures.push(c);
             }
             Macro::Lit(_l) => {
                 self.snippets.push('\n');
-                let snip = expand_sub_rust_macro(line_directive(&self.current_path, begin) + extracted, ExpandSubMacroType::Lit)
-                        .map_err(|e| e.add_line(begin.line))?;
+                let snip = expand_sub_rust_macro(
+                    line_directive(&self.current_path, begin) + extracted,
+                    ExpandSubMacroType::Lit,
+                ).map_err(|e| e.add_line(begin.line))?;
                 self.snippets.push_str(&snip);
             }
         }
@@ -537,10 +576,13 @@ impl<'ast> Visit<'ast> for Parser {
                     ident: ref id,
                     lit: syn::Lit::Str(ref s),
                     ..
-                })) if id == "path" =>
+                }))
+                    if id == "path" =>
                 {
                     let mod_path = self.mod_dir.join(&s.value());
-                    return self.parse_mod(mod_path).unwrap_or_else(|err| self.mod_error = Some(err));
+                    return self
+                        .parse_mod(mod_path)
+                        .unwrap_or_else(|err| self.mod_error = Some(err));
                 }
                 _ => {}
             }
@@ -550,11 +592,28 @@ impl<'ast> Visit<'ast> for Parser {
         let mut subdir = self.mod_dir.join(mod_name.clone());
         subdir.push("mod.rs");
         if subdir.is_file() {
-            return self.parse_mod(subdir).unwrap_or_else(|err| self.mod_error = Some(err));
+            return self
+                .parse_mod(subdir)
+                .unwrap_or_else(|err| self.mod_error = Some(err));
         }
+
+        let mut adjacent_subdir = self.current_path.clone();
+        if let Some(cur_mod_dir) = adjacent_subdir.file_stem().map(|x| x.to_owned()) {
+            adjacent_subdir.pop();
+            adjacent_subdir.push(cur_mod_dir);
+            adjacent_subdir.push(format!("{}.rs", mod_name));
+            if adjacent_subdir.is_file() {
+                return self
+                    .parse_mod(adjacent_subdir)
+                    .unwrap_or_else(|err| self.mod_error = Some(err));
+            }
+        }
+
         let adjacent = self.mod_dir.join(&format!("{}.rs", mod_name));
         if adjacent.is_file() {
-            return self.parse_mod(adjacent).unwrap_or_else(|err| self.mod_error = Some(err));
+            return self
+                .parse_mod(adjacent)
+                .unwrap_or_else(|err| self.mod_error = Some(err));
         }
 
         panic!(
